require "cpay/version"

module Cpay
  require 'json'
  require 'net/http'
  require 'openssl'
  require 'base64'
  require 'cgi'


  module Utils
    extend self

    def rsa_sign(data, private_key)
      pri = OpenSSL::PKey::RSA.new private_key
      sign = pri.sign('sha1', data.force_encoding('utf-8'))
      Base64.encode64(sign).delete("\n").delete("\r")
    end

    def rsa_verify(data, sign, public_key)
      pub = OpenSSL::PKey::RSA.new public_key
      digester = OpenSSL::Digest::SHA1.new
      sign = Base64.decode64(sign)
      pub.verify(digester, sign, data)
    end

    def rsa_encrypt_by_pub(data, public_key)
      pub = OpenSSL::PKey::RSA.new public_key
      Base64.encode64(pub.public_encrypt(data)).delete("\n").delete("\r")
    end

    def rsa_decrypt_by_pri(data, private_key)
      pri = OpenSSL::PKey::RSA.new private_key
      pri.private_decrypt(Base64.decode64(data))
    end

    def aes_encrypt(data, key)
      cipher = OpenSSL::Cipher.new 'AES-128-ECB'
      cipher.encrypt
      cipher.key = key
      txt = cipher.update(data) << cipher.final
      Base64.encode64(txt).delete("\n").delete("\r")
    end

    def aes_decrypt(data, key)
      aes = OpenSSL::Cipher.new('AES-128-ECB')
      aes.decrypt
      aes.key = key
      aes.update(Base64.decode64(data)) + aes.final
    end

    def url_decode(str)
      str.gsub!(/%[a-fA-F0-9]{2}/) {|x| x = x[1..2].hex.chr}
    end

  end

  module Sand
    module Fastpay
      extend self

      def pay(c_params)
        time = Time.now.strftime('%Y%m%d%H%M%S')
        b_params = {
            body: {
                userId: c_params[:userid].to_s, #用户ID
                orderCode: c_params[:orderid].to_s, #订单号
                orderTime: time,
                totalAmount: c_params[:amount].to_s.rjust(12, '0'), #订单金额
                subject: c_params[:subject].to_s, #订单标题
                body: c_params[:body].to_s, #订单描述
                currencyCode: 156,
                notifyUrl: c_params[:notifyurl].to_s, #异步通知地址
                frontUrl: c_params[:fronturl].to_s, #前端跳转地址
            },
            method: "sandPay.fastPay.quickPay.index"
        }
        res = base_pay_act b_params, c_params, true

        if res[:code] = 0
          return %Q{
      <form action="https://cashier.sandpay.com.cn/fastPay/quickPay/index" method="post" hidden="hidden">
      <textarea name="charset">#{res[:data][:charset]}</textarea>
      <textarea name="signType">#{res[:data][:signType]}</textarea>
      <textarea name="data">#{res[:data][:data]}</textarea>
      <textarea name="sign">#{res[:data][:sign]}</textarea>
      </form>
      <script type="text/javascript">
        document.forms[0].submit();
      </script>
      }
        else
          return "生成订单失败，请稍后重试"
        end

      end

      def notify(c_params)
        sign = c_params[:sign]
        data = c_params[:data]
        publickey = c_params[:publickey]
        ret = {
            code: -1,
            msg: '',
            data: {}
        }
        begin
          if Utils.rsa_verify(data, sign, publickey)
            data = JSON data
            ret[:data] = data
            ret[:code] = 0

            ret[:cpay] = {
                ok: data['body']['orderStatus'] == '1',
                msg: '',
                next: data['head']['respCode'] != '000000',
            }
          else
            ret[:code] = 2
          end
        rescue Exception => ex
          ret[:msg] = ex.to_s
          ret[:code] = -1
        end
        ret
      end

      def queryOrder(c_params)
        b_params = {
            body: {
                orderCode: c_params[:orderid].to_s, #订单号
            },
            method: "sandpay.trade.query",
            uri: URI("https://cashier.sandpay.com.cn/gateway/api/order/query")

        }
        ret = base_pay_act b_params, c_params, false
        ret[:cpay] = {
            next: ret[:data]['head']['respCode'] == '030096',
            ok: ret[:data]['body']['oriRespCode'] == '000000' && ret[:data]['body']['orderStatus'] == '00',
            msg: ret[:data]['oriRespMsg']
        }
        ret
      end

      private

      def base_pay_act(b_params, c_params, return_req_data)
        ret = {
            code: -1,
            msg: '',
            data: {}
        }
        begin
          time = Time.now.strftime('%Y%m%d%H%M%S')
          data = {
              head: {
                  version: '1.0',
                  method: b_params[:method],
                  productId: '00000016',
                  accessType: '1',
                  mid: c_params[:mid].to_s,
                  channelType: '07',
                  reqTime: time
              },
              body: b_params[:body]
          }

          req_data = {
              charset: 'utf-8',
              signType: '01',
              data: data.to_json,
              sign: Utils.rsa_sign(data.to_json, c_params[:privatekey])
          }

          if return_req_data
            ret[:code] = 0
            ret[:data] = req_data
          else
            res = Net::HTTP.post_form b_params[:uri], req_data

            puts res.body
            cgi_parse = CGI.parse(Utils.url_decode(res.body))
            unless Utils.rsa_verify(cgi_parse['data'][0], cgi_parse['sign'][0].to_s.gsub(' ', '+'), c_params[:publickey])
              ret[:code] = 2
              ret[:msg] = "签名错误"
            else
              ret[:code] = 0
              ret[:data] = JSON(cgi_parse['data'][0])
            end
          end
        rescue Exception => ex
          ret[:msg] = ex.to_s
          ret[:code] = -1
        end
        ret
      end

    end

    module Draw
      extend self

      def agentpay(c_params)
        b_params = {
            uri: URI("https://caspay.sandpay.com.cn/agent-main/openapi/agentpay"),
            pt: {
                orderCode: c_params[:orderid].to_s,
                version: '01',
                productId: '00000004',
                tranTime: c_params[:tranTime],
                tranAmt: c_params[:amount].to_s.rjust(12, '0'),
                currencyCode: '156',
                accAttr: '0',
                accType: '4',
                accNo: c_params[:accno],
                accName: c_params[:accname],
                remark: c_params[:remark],
            },
            transCode: 'RTPM',
        }

        ret = base_draw_act b_params, c_params
        ret[:cpay] = {
            msg: ret[:data]['respDesc'],
            ok: ret[:code] == 0 && (ret[:data]['resultFlag'] == '0'),
            next: ret[:code] != 0 || ret[:data]['resultFlag'] == '2',
            tranTime: c_params[:tranTime],
            orderid: c_params[:orderid].to_s,
            flag: ret[:data]['resultFlag']
        }
        ret
      end

      def queryBalance(c_params)
        b_params = {
            uri: URI("https://caspay.sandpay.com.cn/agent-main/openapi/queryBalance"),
            pt: {
                orderCode: c_params[:orderid].to_s,
                version: '01',
                productId: '00000004',
                tranTime: Time.now.strftime('%Y%m%d%H%M%S'),
            },
            transCode: 'MBQU',
        }

        ret = base_draw_act b_params, c_params
        ret[:cpay] = {
            ok: ret[:code] == 0 && ret[:data]['respCode'] == '0000',
            msg: ret[:data]['balance'].to_i,
            balance: ret[:data]['balance'].to_i
        }
        ret

      end

      def queryOrder c_params
        b_params = {
            uri: URI("https://caspay.sandpay.com.cn/agent-main/openapi/queryOrder"),
            pt: {
                orderCode: c_params[:orderid].to_s,
                version: '01',
                productId: '00000004',
                tranTime: c_params[:tranTime],
            },
            transCode: 'ODQU',
        }
        ret = base_draw_act b_params, c_params
        ret[:cpay] = {
            ok: ret[:code] == 0 && ret[:data]['resultFlag'] == '0',
            next: ret[:code] != 0 || ret[:data]['resultFlag'] == '2',
            msg: ''
        }
        ret
      end

      private

      def base_draw_act(b_params, c_params)

        ret = {
            code: -1,
            msg: '',
            data: {}
        }
        begin

          uri = b_params[:uri]
          pt = b_params[:pt]
          aes_key = rand(36 ** 16).to_s(36).rjust(16, '0')
          data = {
              transCode: b_params[:transCode],
              accessType: '0',
              merId: c_params[:mid],
              encryptKey: Utils.rsa_encrypt_by_pub(aes_key, c_params[:publickey]),
              encryptData: Utils.aes_encrypt(pt.to_json, aes_key),
              sign: Utils.rsa_sign(pt.to_json, c_params[:privatekey]),
          }
          puts data
          res = Net::HTTP.post_form uri, data
          res_param = {}
          body = res.body
          body[0] = ''
          body[-1] = ''
          Utils.url_decode(body).split('&').each do |str|
            str_split = str.split('=')
            res_param[str_split[0]] = str_split[1]
          end
          res_data = Utils.aes_decrypt(res_param['encryptData'], Utils.rsa_decrypt_by_pri(res_param['encryptKey'], c_params[:privatekey]))

          if Utils.rsa_verify(res_data, res_param['sign'], c_params[:publickey])
            ret[:code] = 0
            ret[:data] = JSON(res_data)
          else
            ret[:code] = 2
            ret[:msg] = "签名错误"
          end
        rescue Exception => ex
          ret[:msg] = ex.to_s
          ret[:code] = -1
        end
        ret
      end
    end
  end


end

